# frozen_string_literal: true
require 'securerandom'
require 'connection_pool'
require 'faraday'
require 'faraday_middleware'
require 'wisper'
require 'horn_of_plenty/adapters/github/configuration'
require 'horn_of_plenty/adapters/github/error'

module  HornOfPlenty
module  Adapters
module  Github
class   Client
  include Wisper::Publisher

  RETRY_BACKOFF_MULTIPLIER = 3.5

  attr_accessor :url,
                :version,
                :token,
                :logger

  def initialize(url:     configuration.base_url,
                 version: configuration.api_version,
                 token:   configuration.token,
                 logger:  configuration.logger)

    self.url     = url
    self.version = version
    self.token   = token
    self.logger  = logger
  end

  # rubocop:disable Metrics/LineLength
  def request(request, retries = 3, _retry_delay_in_ms = 1000)
    broadcast(:horn_of_plenty_request_sent, request.class.name, request.method, request.path, request.to_h, url, retries)

    response = connection.public_send(request.method, request.path, request.to_h)

    broadcast(:horn_of_plenty_response_received, response.class.name, request.method, request.path, response.body, url)

    request.response_class.parse(response)
  rescue Github::Errors => exception
    retries -= 1

    broadcast(:horn_of_plenty_request_retried, exception, retries)

    retry if retries.positive?

    raise exception
  end
  # rubocop:enable Metrics/LineLength

  def self.pool
    @pool ||= ConnectionPool::Wrapper.new(size: 7, timeout: 20) { new }
  end

  def self.configuration
    Github::Configuration.instance
  end

  private

  def configuration
    @configuration ||= self.class.configuration
  end

  def connection
    @connection ||= Faraday.new(url: url) do |conn|
      conn.response :follow_redirects

      conn.adapter :net_http

      conn.headers['Accept']        = "application/vnd.github.#{version}+json"
      conn.headers['User-Agent']    = 'Horn-Of-Plenty-Gem'
      conn.headers['Authorization'] = "Token #{token}"
    end
  end
end
end
end
end
