# frozen_string_literal: true
require 'forwardable'

module  HornOfPlenty
class   Paginator
  include Enumerable
  extend  Forwardable

  def_delegators :items,
                 :length,
                 :size,
                 :count,
                 :empty?,
                 :[],
                 :first

  def initialize(client:,
                 request:,
                 items_per_page: nil,
                 current_page:   nil,
                 autopage:       true)

    self.client         = client
    self.request        = request

    self.items_per_page = items_per_page || 500
    self.current_page   = current_page   || 1
    self.autopage       = autopage
  end

  def all
    items
  end

  def each(_options = {})
    loop do
      items.each do |item|
        yield item
      end

      break unless autopage && go_to_next_page
    end
  end

  def each_page
    loop do
      yield items

      break unless go_to_next_page
    end
  end

  def go_to_next_page
    return false if items.count < items_per_page

    go_to_page(current_page + 1)
  end

  def go_to_previous_page
    go_to_page(current_page - 1)
  end

  def go_to_page(page_number)
    page_number = 1 if page_number < 1

    self.current_page = page_number
  end

  def items
    @items ||= retrieval_result.result
  end

  protected

  attr_accessor :client,
                :request,
                :autopage
  attr_reader   :items_per_page,
                :current_page,
                :current_offset

  def items_per_page=(value)
    @items_per_page   = value.to_i
    @retrieval_result = nil
  end

  def current_page=(value)
    @current_page     = value.to_i
    @current_offset   = (@current_page - 1) * items_per_page
    @retrieval_result = nil
    @items            = nil
  end

  private

  def retrieval_result
    request.pagination = pagination_parameters

    client.request(request)
  end

  def pagination_parameters
    {
      limit:  items_per_page,
      offset: current_offset,
    }
  end
end
end
