# frozen_string_literal: true
# rubocop:disable Style/OptionHash
module  HornOfPlenty
module  Parsers
module  Hash
  private

  def parse_text(hash, key, options = {}, &user_block)
    parse_key(hash, key, ->(value) { value.to_s }, options, &user_block)
  end

  def parse_html(hash, key, options = {}, &user_block)
    converter = ->(value) { CGI.unescape_html(value.to_s) }

    parse_key(hash, key, converter, options, &user_block)
  end

  def parse_integer(hash, key, options = {}, &user_block)
    parse_key(hash, key, ->(value) { value.to_i }, options, &user_block)
  end

  def parse_float(hash, key, options = {}, &user_block)
    parse_key(hash, key, ->(value) { value.to_f }, options, &user_block)
  end

  def parse_time(hash, key, options = {}, &user_block)
    parse_key(hash, key, ->(value) { Time.iso8601(value).round }, options, &user_block)
  end

  def parse_url(hash, key, options = {}, &user_block)
    parse_key(hash, key, ->(value) { URI.parse(value) }, options, &user_block)
  end

  def parse_phone(hash, key, options = {}, &user_block)
    parse_key(hash, key, ->(value) { value.gsub(/[^x\d]/, '') }, options, &user_block)
  end

  def parse_existance(hash, key, options = {}, &user_block)
    parse_key(hash, key, ->(value) { value }, options, &user_block)
  end

  def parse_boolean(hash, key, options = {}, &user_block)
    converter = lambda do |value|
      if value
        %w{true TRUE True t T yes YES Yes y Y 1}.include?(value) ? true : false
      end
    end

    parse_key(hash, key, converter, options, &user_block)
  end

  def parse_percentage(hash, key, options = {}, &user_block)
    parse_key(hash, key, ->(value) { (value.to_f / 100).round(4) }, options, &user_block)
  end

  # rubocop:disable Metrics/PerceivedComplexity, Metrics/CyclomaticComplexity
  def parse_key(hash, key, converter = nil, options = {})
    default_value = if options[:default_value]
                      options[:default_value]
                    elsif options[:array]
                      []
                    end

    value = hash.dig(*key.split('/:/'))
    value = if value.to_s.empty?
              default_value
            else
              value
            end
    value = if options[:array] && value.respond_to?(:split)
              value.split(',')
            else
              value
            end
    value = if value.respond_to?(:map) && converter
              value.map do |item|
                converter.call(item)
              end
            elsif value && converter
              converter.call(value)
            else
              value
            end

    if block_given? && (value || options[:yield_if_nil])
      yield value
    else
      value
    end
  end
  # rubocop:enable Metrics/PerceivedComplexity, Metrics/CyclomaticComplexity
end
end
end
# rubocop:enable Style/OptionHash
